// Copyright (c) 2015, Alexandre Ardhuin
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

@JS()
library google_maps.core;

import 'dart:async' show StreamController;
import 'dart:html' show Document, Element, HtmlElement;
import 'dart:typed_data' show Float32List, Float64List;
import 'dart:web_gl' show RenderingContext;
import 'package:yjy_google_maps/google_maps_places.dart' show PlacePlusCode;
import 'package:js_wrapping/js_wrapping.dart';

part 'core/g_map.dart';
part 'core/map_options.dart';
part 'core/map_type_style.dart';
part 'core/map_mouse_event.dart';
part 'core/icon_mouse_event.dart';
part 'core/map_type_id.dart';
part 'core/map_type_registry.dart';
part 'core/map_restriction.dart';
part 'core/traffic_layer.dart';
part 'core/traffic_layer_options.dart';
part 'core/transit_layer.dart';
part 'core/bicycling_layer.dart';
part 'core/camera_options.dart';
part 'core/visible_region.dart';
part 'core/rendering_type.dart';
part 'core/webgl__overlay_view.dart';
part 'core/webgl__draw_options.dart';
part 'core/webgl__state_options.dart';
part 'core/coordinate_transformer.dart';
part 'core/camera_params.dart';
part 'core/lat_lng.dart';
part 'core/lat_lng_literal.dart';
part 'core/lat_lng_bounds.dart';
part 'core/lat_lng_bounds_literal.dart';
part 'core/lat_lng_altitude.dart';
part 'core/lat_lng_altitude_literal.dart';
part 'core/point.dart';
part 'core/size.dart';
part 'core/padding.dart';
part 'core/circle_literal.dart';
part 'core/event.dart';
part 'core/maps_event_listener.dart';
part 'core/mvc_object.dart';
part 'core/mvc_array.dart';
part 'core/error_event.dart';
part 'core/fullscreen_control_options.dart';
part 'core/map_type_control_options.dart';
part 'core/map_type_control_style.dart';
part 'core/motion_tracking_control_options.dart';
part 'core/pan_control_options.dart';
part 'core/rotate_control_options.dart';
part 'core/scale_control_options.dart';
part 'core/scale_control_style.dart';
part 'core/street_view_control_options.dart';
part 'core/zoom_control_options.dart';
part 'core/control_position.dart';
part 'core/marker.dart';
part 'core/marker_options.dart';
part 'core/collision_behavior.dart';
part 'core/icon.dart';
part 'core/marker_label.dart';
part 'core/marker_shape.dart';
part 'core/g_symbol.dart';
part 'core/symbol_path.dart';
part 'core/animation.dart';
part 'core/info_window.dart';
part 'core/info_window_options.dart';
part 'core/info_window_open_options.dart';
part 'core/polyline.dart';
part 'core/polyline_options.dart';
part 'core/icon_sequence.dart';
part 'core/polygon.dart';
part 'core/polygon_options.dart';
part 'core/poly_mouse_event.dart';
part 'core/rectangle.dart';
part 'core/rectangle_options.dart';
part 'core/circle.dart';
part 'core/circle_options.dart';
part 'core/stroke_position.dart';
part 'core/data.dart';
part 'core/data_data_options.dart';
part 'core/data_geo_json_options.dart';
part 'core/data_style_options.dart';
part 'core/data_feature.dart';
part 'core/data_feature_options.dart';
part 'core/data_geometry.dart';
part 'core/data_point.dart';
part 'core/data_multi_point.dart';
part 'core/data_line_string.dart';
part 'core/data_multi_line_string.dart';
part 'core/data_linear_ring.dart';
part 'core/data_polygon.dart';
part 'core/data_multi_polygon.dart';
part 'core/data_geometry_collection.dart';
part 'core/data_mouse_event.dart';
part 'core/data_add_feature_event.dart';
part 'core/data_remove_feature_event.dart';
part 'core/data_set_geometry_event.dart';
part 'core/data_set_property_event.dart';
part 'core/data_remove_property_event.dart';
part 'core/overlay_view.dart';
part 'core/map_panes.dart';
part 'core/map_canvas_projection.dart';
part 'core/kml_layer.dart';
part 'core/kml_layer_options.dart';
part 'core/kml_layer_metadata.dart';
part 'core/kml_layer_status.dart';
part 'core/kml_mouse_event.dart';
part 'core/kml_feature_data.dart';
part 'core/kml_author.dart';
part 'core/map_type.dart';
part 'core/projection.dart';
part 'core/image_map_type.dart';
part 'core/image_map_type_options.dart';
part 'core/ground_overlay.dart';
part 'core/ground_overlay_options.dart';
part 'core/styled_map_type.dart';
part 'core/styled_map_type_options.dart';
part 'core/max_zoom_service.dart';
part 'core/max_zoom_result.dart';
part 'core/max_zoom_status.dart';
part 'core/street_view_panorama.dart';
part 'core/street_view_panorama_options.dart';
part 'core/street_view_address_control_options.dart';
part 'core/pano_provider_options.dart';
part 'core/street_view_tile_data.dart';
part 'core/street_view_pov.dart';
part 'core/street_view_coverage_layer.dart';
part 'core/street_view_service.dart';
part 'core/street_view_status.dart';
part 'core/street_view_location_request.dart';
part 'core/street_view_pano_request.dart';
part 'core/street_view_response.dart';
part 'core/street_view_location.dart';
part 'core/street_view_preference.dart';
part 'core/street_view_source.dart';
part 'core/street_view_panorama_data.dart';
part 'core/street_view_link.dart';
part 'core/geocoder.dart';
part 'core/geocoder_request.dart';
part 'core/geocoder_component_restrictions.dart';
part 'core/geocoder_status.dart';
part 'core/geocoder_response.dart';
part 'core/geocoder_result.dart';
part 'core/geocoder_address_component.dart';
part 'core/geocoder_geometry.dart';
part 'core/geocoder_location_type.dart';
part 'core/directions_service.dart';
part 'core/directions_request.dart';
part 'core/directions_status.dart';
part 'core/directions_result.dart';
part 'core/directions_renderer.dart';
part 'core/directions_renderer_options.dart';
part 'core/directions_waypoint.dart';
part 'core/directions_geocoded_waypoint.dart';
part 'core/directions_route.dart';
part 'core/directions_leg.dart';
part 'core/directions_step.dart';
part 'core/directions_polyline.dart';
part 'core/place.dart';
part 'core/travel_mode.dart';
part 'core/driving_options.dart';
part 'core/traffic_model.dart';
part 'core/transit_options.dart';
part 'core/transit_mode.dart';
part 'core/transit_route_preference.dart';
part 'core/transit_fare.dart';
part 'core/transit_details.dart';
part 'core/transit_stop.dart';
part 'core/transit_line.dart';
part 'core/transit_agency.dart';
part 'core/transit_vehicle.dart';
part 'core/vehicle_type.dart';
part 'core/unit_system.dart';
part 'core/distance.dart';
part 'core/g_duration.dart';
part 'core/time.dart';
part 'core/distance_matrix_service.dart';
part 'core/distance_matrix_request.dart';
part 'core/distance_matrix_response.dart';
part 'core/distance_matrix_response_row.dart';
part 'core/distance_matrix_response_element.dart';
part 'core/distance_matrix_status.dart';
part 'core/distance_matrix_element_status.dart';
part 'core/elevation_service.dart';
part 'core/location_elevation_request.dart';
part 'core/location_elevation_response.dart';
part 'core/path_elevation_request.dart';
part 'core/path_elevation_response.dart';
part 'core/elevation_result.dart';
part 'core/elevation_status.dart';
part 'core/settings.dart';
part 'core/maps_network_error.dart';
part 'core/maps_request_error.dart';
part 'core/maps_server_error.dart';
part 'core/maps_network_error_endpoint.dart';
